api-service:
---------------


IChassisPlugin
--------------

All plugins are installed immediately after the Chassis boots() but before it starts().

An IChassisPlugin receives the IChassisContext instance and it's own options.

A IChassisContext may bind to Express or the Event bus or something else entirely.

```
export interface IChassisPlugin extends IChassisExtension {

    name: string;
    title?: string;
    defaults?: any

    install (context: IChassisContext, options: any) :void ;

}
```

IChassisMiddleware
------------------

The IChassisMiddleware uses a factory pattern to instantiate Express/Connect middleware .

Each IChassisMiddleware is initialized with an IChassisContext and it's own options.

The purpose of IChassisMiddleware is to returns an Express/Connect middleware with the typical (req,res, next) signature.

The OpenAPI plugin parses the specification and uses the chassis.operationId to inject the feature:

```
    feature:
        example:
            option_a: true
            option_b: 42

````

To create a new middleware feature, the `IChassisMiddleware` interface must be implemented. The `fn()` returns the de-facto standard `express connect` middleware.

```
export interface IChassisMiddleware extends IChassisExtension {
    name: string;
    title?: string;
    defaults?: any

    fn (oas_operation: IOperation, options: any) : Handler|Function ;
}
```

IChassisContext
---------------

The context object contains essential system bindings.

```
    context.api         the Express engine - use to bind routes in IChassisPlugin.
    context.bus         the internal EventEmitter. 
    context.log         writes to log via the Winston "logger".
    context.audit       writes to audit log via the Winston "auditor".
    context.openapi     bound when "openapi" plugin successfully installs.
```
