"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Vars_1 = require("./Vars");
const _ = require("lodash");
// import * as http from "http";
// import * as https from "https";
// import { URL } from "url";
const axios_1 = require("axios");
class APIs {
    constructor() { }
    static get(context, url) {
        return new Promise((resolve, reject) => {
            axios_1.default.get(url)
                .then((rep) => {
                resolve(rep.data);
            })
                .catch((err) => {
                context.error({
                    code: "feature:request:get:failed",
                    url: url,
                    error: err ? err.toString() : "no error",
                });
                reject(err);
            });
        });
    }
    static request(context, url, _headers) {
        // let headers = APIs.get_headers_with_token(_headers, token);
        return new Promise((resolve, reject) => {
            // call backend API
            APIs.DEBUG &&
                context.log({
                    code: "feature:request:api",
                    url: url,
                });
            // call backend API
            APIs.DEBUG &&
                context.log({
                    code: "feature:request:api",
                    url: url,
                });
            axios_1.default.get(url, { headers: _headers })
                .then((rep) => {
                resolve(rep.data);
            })
                .catch((err) => {
                context.error({
                    code: "feature:request:api:failed",
                    url: url,
                    error: err ? err.toString() : "no error",
                });
                reject(err);
            });
        });
    }
    static named_request(context, api_name, 
    // url: string,
    api) {
        return new Promise((resolve, reject) => {
            // call backend API
            APIs.DEBUG &&
                context.log({
                    code: "feature:api:request",
                    api: api_name,
                    url: api.url,
                    headers: api.headers,
                });
            axios_1.default.request({
                url: api.url,
                headers: api.headers,
                params: api.params,
            })
                .then((resp) => {
                try {
                    let result = {};
                    APIs.DEBUG &&
                        context.log({
                            code: "feature:api:response",
                            api: api_name,
                            url: api.url,
                            response: resp,
                        });
                    result[api_name] = _.isObject(resp.data)
                        ? resp.data
                        : JSON.parse(resp.data);
                    resolve(result);
                }
                catch (e) {
                    console.error("feature:api:json:failed: %o --> %o", e, resp);
                    reject(e);
                }
            })
                .catch((err) => {
                context.error({
                    code: "feature:api:failed",
                    api: api_name,
                    url: api.url,
                    error: err ? err.toString() : "no error",
                });
                reject(err);
            });
        });
    }
    static get_headers_with_token(headers, token) {
        let _headers = _.extend({}, headers);
        if (token) {
            // inject Bearer Token
            _headers.Authorization = token;
        }
        return _headers;
    }
    /**
     * Collect a set of API responses as promise
     *
     * @param apis
     * @param opx_ctx
     * @param token
     */
    static requests(context, apis, op_ctx, token) {
        let _requests = [];
        // iterate over set of APIs
        for (let api_name in apis) {
            let api = _.extend({}, apis[api_name]);
            api.headers = APIs.get_headers_with_token(api.headers, token);
            api.params = _.extend({}, api.params);
            if (token) {
                // inject Bearer Token
                api.headers.authorization = token;
            }
            api.url = Vars_1.default.$$(api.url, op_ctx);
            let _request = this.named_request(context, api_name, api);
            _requests.push(_request);
        }
        return new Promise((resolve, reject) => {
            // concurrent "named" requests
            Promise.all(_requests)
                .then((responses) => {
                // merge named results
                let results = {};
                for (let r in responses) {
                    _.extend(results, responses[r]);
                }
                resolve(results);
            })
                .catch(reject);
        });
    }
    static getOperationContext(_op, req, _ctx) {
        let ctx = _.extend({}, _ctx, {
            op: {
                resource: _op.resource,
                actionId: _op.actionId,
                operationId: _op.operationId,
            },
        });
        return APIs.getRequestContext(req, ctx);
    }
    static getRequestContext(req, _ctx) {
        let params = _.extend({}, req.query, req.params); // merged params
        let ctx = _.extend({ params: {}, jwt: {}, query: {} }, _ctx, req
            ? {
                req: { path: req.path, method: req.method },
                params: params,
                path: req.params,
                query: req.query,
                jwt: req.jwt || {},
            }
            : {});
        // console.log("req.ctx: %o --> %o", req.params, ctx);
        return ctx;
    }
    static resource_path_to_express_path(resource) {
        return resource
            .replace(new RegExp("{", "g"), ":")
            .replace(new RegExp("}", "g"), "");
    }
    static getContentType(type) {
        if (!type)
            return "application/json";
        let ix = type.indexOf(";");
        if (ix < 0)
            return type;
        return type.substring(0, ix);
    }
    static oops(context, res, log) {
        context.warn(log);
        res.status(log.statusCode || 400);
        res.send({
            code: log.code,
            message: log.message || log.error || log.code,
            error: log.error || true,
        });
    }
    static sleep(ms) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => setTimeout(resolve, ms));
        });
    }
}
exports.default = APIs;
APIs.DEBUG = false;
//# sourceMappingURL=APIs.js.map